from twisted.protocols import amp

# If it inherits from Server2Client, it's a message directed at clients.
Server2Client = amp.Command
# If it inherits from Client2Server, it's a message directed at servers.
Client2Server = amp.Command
# If it inherits from Bidirectional, it does exactly what that says. :)
Bidirectional = amp.Command

class Genesis(Server2Client):
    """Tell the client what the world looks like"""
    requiresAnswer = False
    arguments = [
        ('prng_seed', amp.Integer()),
        ('player_names', amp.ListOf(amp.String())),
        ('player_id', amp.Integer()),
        ##('blocking', amp.ListOf(amp.Integer())),
        ##('enemy_ids', amp.ListOf(amp.Integer()))
        # TODO: add ways to create different types of enemies
    ]
        
    response = []
    errors = {}

class Update(Server2Client):
    """Update the entire state of the world!"""
    requiresAnswer = False
    arguments = [
        ('kill_ids', amp.ListOf(amp.Integer())),
        ('move_ids', amp.ListOf(amp.ListOf(amp.Integer())))] # format: [id, x, y]
    response = []
    errors = {}

class UserReady(Server2Client):
    """
    user is ready to play!
    """
    requiresAnswer = False
    arguments = [
        ('username', amp.String())]
    response = [] # TODO: nick validation [no duplicates]
    errors = {}

class IAmReady(Client2Server):
    """I am ready to play!"""
    requiresAnswer = False
    arguments = []
    response = []
    errors = {}

class Login(Bidirectional):
    """
    C2S: please log me in
    S2C: this user logged in
    """
    requiresAnswer = False
    arguments = [
        ('username', amp.String())]
    response = [] # TODO: nick validation [no duplicates]
    errors = {}


class TryMove(Client2Server):
    """Send a movement attempt!"""
    requiresAnswer = False
    arguments = [
        ('char_id', amp.Integer()),
        ('new_x', amp.Integer()),
        ('new_y', amp.Integer())]
    response = []
    errors = {}

class Kill(Client2Server):
    requiresAnswer = False
    arguments = [('kill_ids', amp.ListOf(amp.Integer()))]
    response = []
    errors = {}
